package com.diagbot.aggregate;

import com.diagbot.dto.NumDTO;
import com.diagbot.facade.BehospitalInfoFacade;
import com.diagbot.facade.QcCasesFacade;
import com.diagbot.facade.QcresultInfoFacade;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.FilterVO;
import com.diagbot.vo.QcresultFilterVO;
import com.google.common.collect.Lists;
import io.github.lvyahui8.spring.annotation.DataConsumer;
import io.github.lvyahui8.spring.annotation.DataProvider;
import io.github.lvyahui8.spring.annotation.InvokeParameter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:缺陷相关统计
 * @author: gaodm
 * @time: 2020/4/22 18:09
 */
@Component
public class ResultStatisticsAggregate {
    @Autowired
    private QcresultInfoFacade qcresultInfoFacade;
    @Autowired
    private QcCasesFacade qcCasesFacade;
    @Autowired
    private BehospitalInfoFacade behospitalInfoFacade;

    @DataProvider("setAllResult")
    public Map<String, Object> setAllResult(
            @InvokeParameter("filterVO") FilterVO filterVO,
            @DataConsumer("entryByDept") List<NumDTO> deptList,
            @DataConsumer("entryCountGroupByEntry") List<NumDTO> entryList,
            @DataConsumer("entryCountGroupByCase") List<NumDTO> caseList) {
        Map<String, Object> retMap = new LinkedHashMap<>();
        retMap.put("各科室缺陷占比", Lists.newLinkedList());
        retMap.put("各模块缺陷占比排行", Lists.newLinkedList());
        retMap.put("条目缺陷占比", Lists.newLinkedList());
        if (ListUtil.isNotEmpty(deptList)) {
            retMap.put("各科室缺陷占比", deptList);
        }
        if (ListUtil.isNotEmpty(caseList)) {
            retMap.put("各模块缺陷占比排行", caseList);
        }
        if (ListUtil.isNotEmpty(entryList)) {
            retMap.put("条目缺陷占比", entryList);
        }

        return retMap;
    }

    /**
     * 各模块缺陷占比排行
     *
     * @param filterVO
     * @return
     */
    @DataProvider("entryCountGroupByCase")
    public List<NumDTO> entryCountGroupByCase(@InvokeParameter("filterVO") FilterVO filterVO) {
        Integer limitCount = filterVO.getLimitCount();
        QcresultFilterVO qcresultFilterVO = new QcresultFilterVO();
        BeanUtil.copyProperties(filterVO, qcresultFilterVO);
        int mrNum = qcresultInfoFacade.resultCount(qcresultFilterVO);
        List<NumDTO> qcEntryNumList = qcresultInfoFacade.entryCountGroupByCase(filterVO);
        List<NumDTO> standardEntryNumList = qcCasesFacade.entryGroupByCase();
        if (ListUtil.isEmpty(qcEntryNumList)) {
            standardEntryNumList.forEach(entryNum -> {
                Integer totleNum = entryNum.getNum() * mrNum;
                entryNum.setNum(0);
                entryNum.setPercent(0d);
                entryNum.setPercentStr("0%");
                entryNum.setTotleNum(totleNum);
            });
        } else {
            Map<Long, Integer> qcEntryNumMap
                    = EntityUtil.makeMapWithKeyValue(qcEntryNumList, "id", "num");
            if (ListUtil.isNotEmpty(standardEntryNumList)) {
                standardEntryNumList.forEach(entryNum -> {
                    Integer totleNum = entryNum.getNum() * mrNum;
                    if (qcEntryNumMap.containsKey(entryNum.getId())) {
                        entryNum.setNum(qcEntryNumMap.get(entryNum.getId()));
                        Double percent = BigDecimal.valueOf(entryNum.getNum())
                                .divide(BigDecimal.valueOf(totleNum), 4, RoundingMode.HALF_UP)
                                .doubleValue();
                        DecimalFormat df = new DecimalFormat("#0.00");
                        String percentStr
                                = df.format(BigDecimal.valueOf(percent).multiply(BigDecimal.valueOf(100))) + "%";
                        entryNum.setTotleNum(totleNum);
                        entryNum.setPercent(percent);
                        entryNum.setPercentStr(percentStr);
                    } else {
                        entryNum.setNum(0);
                        entryNum.setPercent(0d);
                        entryNum.setPercentStr("0%");
                        entryNum.setTotleNum(totleNum);
                    }
                });
            }
            //降序排序
            Collections.sort(standardEntryNumList, new Comparator<NumDTO>() {
                @Override
                public int compare(NumDTO o1, NumDTO o2) {
                    return o2.getPercent().compareTo(o1.getPercent());
                }
            });
        }

        //取top10
        standardEntryNumList = standardEntryNumList
                .stream()
                .limit(limitCount)
                .collect(Collectors.toList());

        return standardEntryNumList;
    }

    /**
     * 条目缺陷占比
     *
     * @param filterVO
     * @return
     */
    @DataProvider("entryCountGroupByEntry")
    public List<NumDTO> entryCountGroupByEntry(@InvokeParameter("filterVO") FilterVO filterVO) {
        DecimalFormat df = new DecimalFormat("#0.00");
        Integer limitCount = filterVO.getLimitCount();
        List<NumDTO> numDTOList = qcresultInfoFacade.entryCountGroupByEntry(filterVO);
        if (ListUtil.isNotEmpty(numDTOList)) {
            int totle = numDTOList
                    .stream()
                    .map(NumDTO::getNum)
                    .reduce(0, Integer::sum);
            List<NumDTO> retList = Lists.newLinkedList();
            numDTOList.forEach(numDTO -> {
                numDTO.setTotleNum(totle);
                Double percent = BigDecimal.valueOf(numDTO.getNum())
                        .divide(BigDecimal.valueOf(totle), 4, RoundingMode.HALF_UP)
                        .doubleValue();
                numDTO.setPercent(percent);
                String percentStr
                        = df.format(BigDecimal.valueOf(percent).multiply(BigDecimal.valueOf(100))) + "%";
                numDTO.setPercentStr(percentStr);
            });

            //降序排序
            Collections.sort(numDTOList, new Comparator<NumDTO>() {
                @Override
                public int compare(NumDTO o1, NumDTO o2) {
                    return o2.getPercent().compareTo(o1.getPercent());
                }
            });

            //top9+其他
            if (numDTOList.size() <= limitCount) {
                retList.addAll(numDTOList);
            } else {
                int count = 0;
                double sumPercent = 0d;
                for (NumDTO numDTO : numDTOList) {
                    if (retList.size() < limitCount - 1) {
                        retList.add(numDTO);
                        count += numDTO.getNum();
                        sumPercent = BigDecimal.valueOf(sumPercent)
                                .add(BigDecimal.valueOf(numDTO.getPercent()))
                                .doubleValue();
                    } else {
                        NumDTO otherNumDTO = new NumDTO();
                        int num = totle - count;
                        otherNumDTO.setName("其他");
                        otherNumDTO.setNum(num);
                        otherNumDTO.setTotleNum(totle);
                        Double percent = BigDecimal.valueOf(1)
                                .subtract(BigDecimal.valueOf(sumPercent))
                                .doubleValue();
                        otherNumDTO.setPercent(percent);
                        String percentStr
                                = df.format(BigDecimal.valueOf(percent).multiply(BigDecimal.valueOf(100))) + "%";
                        otherNumDTO.setPercentStr(percentStr);
                        retList.add(otherNumDTO);
                        break;
                    }
                }
            }
            return retList;
        }
        return null;
    }

    /**
     * 各科室缺陷占比
     *
     * @param filterVO
     * @return
     */
    @DataProvider("entryByDept")
    public List<NumDTO> entryByDept(@InvokeParameter("filterVO") FilterVO filterVO) {
        Integer limitCount = filterVO.getLimitCount();
        List<NumDTO> numDTOList = behospitalInfoFacade.resultStatisticsByDept2(filterVO);
        if (ListUtil.isNotEmpty(numDTOList)) {
            int totle = numDTOList
                    .stream()
                    .map(NumDTO::getNum)
                    .reduce(0, Integer::sum);
            numDTOList.forEach(result -> {
                Double percent = BigDecimal.valueOf(result.getNum())
                        .divide(BigDecimal.valueOf(totle), 4, RoundingMode.HALF_UP)
                        .doubleValue();
                result.setPercent(percent);
            });
            List<NumDTO> retList = Lists.newLinkedList();
            if (numDTOList.size() <= limitCount) {
                retList = BeanUtil.listCopyTo(numDTOList, NumDTO.class);
            } else {

                Double rate = 0d;
                Integer num = 0;
                for (NumDTO numDTO : numDTOList) {
                    if (retList.size() < limitCount - 1) {
                        rate = BigDecimal.valueOf(rate)
                                .add(BigDecimal.valueOf(Double.valueOf(numDTO.getPercent())))
                                .doubleValue();
                        retList.add(numDTO);
                    } else {
                        num += numDTO.getNum();
                    }
                }
                NumDTO otherNumDTO = new NumDTO();
                otherNumDTO.setName("其他");
                otherNumDTO.setNum(num);
                otherNumDTO.setPercent(BigDecimal.valueOf(1).subtract(BigDecimal.valueOf(rate)).doubleValue());
                retList.add(otherNumDTO);
            }
            retList.forEach(result -> {
                DecimalFormat df = new DecimalFormat("#0.00");
                String percentStr
                        = df.format(BigDecimal.valueOf(result.getPercent()).multiply(BigDecimal.valueOf(100))) + "%";
                result.setPercentStr(percentStr);
            });
            return retList;
        }
        return null;
    }
}