package com.diagbot.facade;

import com.diagbot.aggregate.AverageStatisticsAggregate;
import com.diagbot.aggregate.MrStatisticsAggregate;
import com.diagbot.aggregate.ResultStatisticsAggregate;
import com.diagbot.dto.AverageStatisticsDTO;
import com.diagbot.dto.NumDTO;
import com.diagbot.dto.QcResultPercentDTO;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.ListUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.vo.FilterVO;
import io.github.lvyahui8.spring.aggregate.facade.DataBeanAggregateQueryFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/4/13 16:45
 */
@Component
public class ConsoleFacade {
    @Autowired
    private FilterFacade filterFacade;
    @Autowired
    private DataBeanAggregateQueryFacade dataBeanAggregateQueryFacade;

    @Autowired
    private AverageStatisticsAggregate averageStatisticsAggregate;
    @Autowired
    private MrStatisticsAggregate mrStatisticsAggregate;
    @Autowired
    private ResultStatisticsAggregate resultStatisticsAggregate;

    //-----------------------聚合接口开始-------------------------------

    /**
     * 病历相关统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> mrStatistics(FilterVO filterVO) {
        Map<String, Object> retMap = new LinkedHashMap<>();
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String startDate = filterFacade.getStartDateStr(filterVO.getType(), null);
        String endDate = filterFacade.getEndDateStr(filterVO.getType(), null);
        filterVO.setStartDate(startDate);
        filterVO.setEndDate(endDate);
        filterVO.setHospitalId(hospitalId);
        if (filterVO.getLimitCount() == null || filterVO.getLimitCount().equals(0)) {
            filterVO.setLimitCount(10);
        }
        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("filterVO", filterVO);
            retMap
                    = dataBeanAggregateQueryFacade.get("setAllMr", invokeParams, Map.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
        }
        return retMap;
    }

    /**
     * 缺陷相关统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> resultStatistics(FilterVO filterVO) {
        Map<String, Object> retMap = new LinkedHashMap<>();
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String startDate = filterFacade.getStartDateStr(filterVO.getType(), null);
        String endDate = filterFacade.getEndDateStr(filterVO.getType(), null);
        filterVO.setHospitalId(hospitalId);
        filterVO.setStartDate(startDate);
        filterVO.setEndDate(endDate);
        if (filterVO.getLimitCount() == null || filterVO.getLimitCount().equals(0)) {
            filterVO.setLimitCount(10);
        }
        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("filterVO", filterVO);
            retMap
                    = dataBeanAggregateQueryFacade.get("setAllResult", invokeParams, Map.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
        }
        return retMap;
    }

    /**
     * 平局值相关统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> averageStatistics(FilterVO filterVO) {
        Map<String, Object> retMap = new LinkedHashMap<>();
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String startDate = filterFacade.getStartDateStr(filterVO.getType(), null);
        String endDate = filterFacade.getEndDateStr(filterVO.getType(), null);
        filterVO.setHospitalId(hospitalId);
        filterVO.setStartDate(startDate);
        filterVO.setEndDate(endDate);
        if (filterVO.getLimitCount() == null || filterVO.getLimitCount().equals(0)) {
            filterVO.setLimitCount(10);
        }
        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("filterVO", filterVO);
            retMap
                    = dataBeanAggregateQueryFacade.get("setAllAverage", invokeParams, Map.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
        }
        return retMap;
    }

    //-----------------------聚合接口结束-------------------------------

    //-----------------------单独接口开始-------------------------------
    /**
     * 平均住院天数
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> getAverageDayNum(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<AverageStatisticsDTO> averageDayNumList = averageStatisticsAggregate.getAverageDayNum(filterVO);
        if (ListUtil.isNotEmpty(averageDayNumList)) {
            retMap.put("平均住院日", averageDayNumList);
        }
        return retMap;
    }

    /**
     * 平均住院费用
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> getAverageFee(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<AverageStatisticsDTO> averageFeeList = averageStatisticsAggregate.getAverageFee(filterVO);
        if (ListUtil.isNotEmpty(averageFeeList)) {
            retMap.put("平均住院费用", averageFeeList);
        }
        return retMap;
    }

    /**
     * 质控平均分按科室统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> getAverageScore(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<AverageStatisticsDTO> averageScoreList = averageStatisticsAggregate.getAverageScore(filterVO);
        if (ListUtil.isNotEmpty(averageScoreList)) {
            retMap.put("各科室质控平均分", averageScoreList);
        }
        return retMap;
    }

    /**
     * 各科室甲级病历占比
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> getLevelResultDept(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<QcResultPercentDTO> levelResults = mrStatisticsAggregate.getLevelResultDept(filterVO);
        if (ListUtil.isNotEmpty(levelResults)) {
            retMap.put("各科室甲级病历占比", levelResults);
        }
        return retMap;
    }

    /**
     * 出院人数统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> leaveHosCount(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        Map<String, Object> leaveHosMap = mrStatisticsAggregate.leaveHosCount(filterVO);
        if (leaveHosMap != null) {
            retMap.put("出院人数统计", leaveHosMap);
        }
        return retMap;
    }

    /**
     * 质控病历统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> mrCount(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<NumDTO> mrCountList = mrStatisticsAggregate.mrCount(filterVO);
        if (ListUtil.isNotEmpty(mrCountList)) {
            retMap.put("病历数统计", mrCountList);
        }
        return retMap;
    }

    /**
     * 各模块缺陷占比排行
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> entryCountGroupByCase(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<NumDTO> caseList = resultStatisticsAggregate.entryCountGroupByCase(filterVO);
        if (ListUtil.isNotEmpty(caseList)) {
            retMap.put("各模块缺陷占比排行", caseList);
        }
        return retMap;
    }

    /**
     * 条目缺陷占比
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> entryCountGroupByEntry(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<NumDTO> entryList = resultStatisticsAggregate.entryCountGroupByEntry(filterVO);
        if (ListUtil.isNotEmpty(entryList)) {
            retMap.put("条目缺陷占比", entryList);
        }
        return retMap;
    }

    /**
     * 各科室缺陷占比
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> entryByDept(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<NumDTO> deptList = resultStatisticsAggregate.entryByDept(filterVO);
        if (ListUtil.isNotEmpty(deptList)) {
            retMap.put("各科室缺陷占比", deptList);
        }
        return retMap;
    }

    private void filterVOSet(FilterVO filterVO){
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String startDate = filterFacade.getStartDateStr(filterVO.getType(), null);
        String endDate = filterFacade.getEndDateStr(filterVO.getType(), null);
        filterVO.setStartDate(startDate);
        filterVO.setEndDate(endDate);
        filterVO.setHospitalId(hospitalId);
        if (filterVO.getLimitCount() == null || filterVO.getLimitCount().equals(0)) {
            filterVO.setLimitCount(10);
        }
    }
    //-----------------------单独接口结束-------------------------------
}