package com.diagbot.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.diagbot.dto.AverageStatisticsDTO;
import com.diagbot.dto.DeptNumDTO;
import com.diagbot.dto.NumDTO;
import com.diagbot.dto.QcResultPercentDTO;
import com.diagbot.entity.QcresultInfo;
import com.diagbot.vo.FilterByDeptVO;
import com.diagbot.vo.FilterVO;
import com.diagbot.vo.QcresultFilterByDeptVO;
import com.diagbot.vo.QcresultFilterVO;

import java.util.List;

/**
 * <p>
 * 质控评分结果信息
每次评分增加一条信息，前面所有评分is_deleted全部设置为Y Mapper 接口
 * </p>
 *
 * @author zhoutg
 * @since 2020-04-13
 */
public interface QcresultInfoMapper extends BaseMapper<QcresultInfo> {

    /**
     * 筛选质控记录
     *
     * @param qcresultFilterVO
     * @return
     */
    public int getQcresultSelectively(QcresultFilterVO qcresultFilterVO);

    /**
     * 质控病历数统计
     *
     * @param qcresultFilterVO
     * @return
     */
    public int resultCount(QcresultFilterVO qcresultFilterVO);

    /**
     * 按科室统计各等级病历百分比
     *
     * @param filterVO
     * @return
     */
    public List<QcResultPercentDTO> levelPercentGroupByDept(FilterVO filterVO);

    /**
     * 按模块统计缺陷数
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> entryCountGroupByCase(FilterVO filterVO);

    /**
     * 条目缺陷分组统计
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> entryCountGroupByEntry(FilterVO filterVO);

    /**
     * 质控平均分按科室统计
     *
     * @param filterVO
     * @return
     */
    public List<AverageStatisticsDTO> getAverageScoreByDept(FilterVO filterVO);

    /**
     * 按科室统计质控病历数
     *
     * @param qcresultFilterByDeptVO
     * @return
     */
    public List<NumDTO> resultCountByDept(QcresultFilterByDeptVO qcresultFilterByDeptVO);

    /**
     * 各模块缺陷占比-按科室
     *
     * @param filterByDeptVO
     * @return
     */
    public List<DeptNumDTO> entryCountGroupByCaseAndDept(FilterByDeptVO filterByDeptVO);

    /**
     * 条目缺陷占比-按科室
     *
     * @param filterByDeptVO
     * @return
     */
    public List<DeptNumDTO> entryCountGroupByEntryAndDept(FilterByDeptVO filterByDeptVO);
}