package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.ConsoleByDeptFacade;
import com.diagbot.vo.FilterByDeptVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;

/**
 * @author zhaops
 * @Description:
 * @date 2020/5/9 13:20
 **/
@RequestMapping("/consoleByDept")
@RestController
@Api(value = "控制台（科室）统计相关API", tags = { "控制台（科室）统计相关API" })
@SuppressWarnings("unchecked")
public class ConsoleByDeptController {
    @Autowired
    private ConsoleByDeptFacade consoleByDeptFacade;

    @ApiOperation(value = "出院病人统计（科室）[by:zhaops]",
            notes = "type： 统计维度 1-本月，2-本年(必填)<br>")
    @PostMapping("/leaveHosCountByDept")
    @SysLogger("leaveHosCountByDept")
    public RespDTO<Map<String, Object>> leaveHosCountByDept(@RequestBody FilterByDeptVO filterByDeptVO) {
        Map<String, Object> data = consoleByDeptFacade.leaveHosCountByDept(filterByDeptVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "病历数统计（科室）[by:zhaops]",
            notes = "type： 统计维度 1-本月，2-本年(必填)<br>")
    @PostMapping("/mrCountByDept")
    @SysLogger("mrCountByDept")
    public RespDTO<Map<String, Object>> mrCountByDept(@RequestBody FilterByDeptVO filterByDeptVO) {
        Map<String, Object> data = consoleByDeptFacade.mrCountByDept(filterByDeptVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "各模块缺陷占比排名（科室）[by:zhaops]",
            notes = "type： 统计维度 1-本月，2-本年(必填)<br>")
    @PostMapping("/entryCountGroupByCaseAndDept")
    @SysLogger("entryCountGroupByCaseAndDept")
    public RespDTO<Map<String, Object>> entryCountGroupByCaseAndDept(@RequestBody FilterByDeptVO filterByDeptVO) {
        Map<String, Object> data = consoleByDeptFacade.entryCountGroupByCaseAndDept(filterByDeptVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "条目缺陷占比（科室）[by:zhaops]",
            notes = "type： 统计维度 1-本月，2-本年(必填)<br>")
    @PostMapping("/entryCountGroupByEntryAndDept")
    @SysLogger("entryCountGroupByEntryAndDept")
    public RespDTO<Map<String, Object>> entryCountGroupByEntryAndDept(@RequestBody FilterByDeptVO filterByDeptVO) {
        Map<String, Object> data = consoleByDeptFacade.entryCountGroupByEntryAndDept(filterByDeptVO);
        return RespDTO.onSuc(data);
    }
}
