package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.ConsoleFacade;
import com.diagbot.vo.FilterVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import java.util.Map;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/4/13 16:45
 */
@RequestMapping("/console")
@RestController
@Api(value = "控制台统计相关API", tags = { "控制台统计相关API" })
@SuppressWarnings("unchecked")
public class ConsoleController {

    @Autowired
    private ConsoleFacade consoleFacade;

    //-----------------------聚合接口开始-------------------------------
    @ApiOperation(value = "病历相关统计[by:zhaops]",
            notes = "type： 统计维度 1-本月，2-本年(必填)<br>")
    @PostMapping("/mrStatistics")
    @SysLogger("mrStatistics")
    @ApiIgnore
    public RespDTO<Map<String, Object>> mrStatistics(@RequestBody FilterVO filterVO) {
        Map<String, Object> data = consoleFacade.mrStatistics(filterVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "缺陷相关统计[by:zhaops]",
            notes = "type： 统计维度 1-本月，2-本年(必填)<br>")
    @PostMapping("/resultStatistics")
    @SysLogger("resultStatistics")
    @ApiIgnore
    public RespDTO<Map<String, Object>> resultStatistics(@RequestBody FilterVO filterVO) {
        Map<String, Object> data = consoleFacade.resultStatistics(filterVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "平均值相关统计[by:zhaops]",
            notes = "type： 统计维度 1-本月，2-本年(必填)<br>")
    @PostMapping("/averageStatistics")
    @SysLogger("averageStatistics")
    @ApiIgnore
    public RespDTO<Map<String, Object>> averageStatistics(@RequestBody FilterVO filterVO) {
        Map<String, Object> data = consoleFacade.averageStatistics(filterVO);
        return RespDTO.onSuc(data);
    }
    //-----------------------聚合接口结束-------------------------------

    //-----------------------单独接口开始-------------------------------

    /**
     * 平均住院天数
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "平均住院天数[by:zhaops]",
            notes = "type： 统计维度 1-本月，2-本年(必填)<br>")
    @PostMapping("/getAverageDayNum")
    @SysLogger("getAverageDayNum")
    public RespDTO<Map<String, Object>> getAverageDayNum(@RequestBody FilterVO filterVO) {
        return RespDTO.onSuc(consoleFacade.getAverageDayNum(filterVO));
    }

    /**
     * 平均住院费用
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "平均住院费用[by:zhaops]",
            notes = "type： 统计维度 1-本月，2-本年(必填)<br>")
    @PostMapping("/getAverageFee")
    @SysLogger("getAverageFee")
    public RespDTO<Map<String, Object>> getAverageFee(@RequestBody FilterVO filterVO) {
        return RespDTO.onSuc(consoleFacade.getAverageFee(filterVO));
    }

    /**
     * 质控平均分按科室统计
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "质控平均分按科室统计[by:zhaops]",
            notes = "type： 统计维度 1-本月，2-本年(必填)<br>")
    @PostMapping("/getAverageScore")
    @SysLogger("getAverageScore")
    public RespDTO<Map<String, Object>> getAverageScore(@RequestBody FilterVO filterVO) {
        return RespDTO.onSuc(consoleFacade.getAverageScore(filterVO));
    }

    /**
     * 各科室甲级病历占比
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "各科室甲级病历占比[by:zhaops]",
            notes = "type： 统计维度 1-本月，2-本年(必填)<br>")
    @PostMapping("/getLevelResultDept")
    @SysLogger("getLevelResultDept")
    public RespDTO<Map<String, Object>> getLevelResultDept(@RequestBody FilterVO filterVO) {
        return RespDTO.onSuc(consoleFacade.getLevelResultDept(filterVO));
    }

    /**
     * 出院人数统计
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "出院人数统计[by:zhaops]",
            notes = "type： 统计维度 1-本月，2-本年(必填)<br>")
    @PostMapping("/leaveHosCount")
    @SysLogger("leaveHosCount")
    public RespDTO<Map<String, Object>> leaveHosCount(@RequestBody FilterVO filterVO) {
        return RespDTO.onSuc(consoleFacade.leaveHosCount(filterVO));
    }

    /**
     * 质控病历统计
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "质控病历统计[by:zhaops]",
            notes = "type： 统计维度 1-本月，2-本年(必填)<br>")
    @PostMapping("/mrCount")
    @SysLogger("mrCount")
    public RespDTO<Map<String, Object>> mrCount(@RequestBody FilterVO filterVO) {
        return RespDTO.onSuc(consoleFacade.mrCount(filterVO));
    }

    /**
     * 各模块缺陷占比排行
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "各模块缺陷占比排行[by:zhaops]",
            notes = "type： 统计维度 1-本月，2-本年(必填)<br>")
    @PostMapping("/entryCountGroupByCase")
    @SysLogger("entryCountGroupByCase")
    public RespDTO<Map<String, Object>> entryCountGroupByCase(@RequestBody FilterVO filterVO) {
        return RespDTO.onSuc(consoleFacade.entryCountGroupByCase(filterVO));
    }

    /**
     * 条目缺陷占比
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "条目缺陷占比[by:zhaops]",
            notes = "type： 统计维度 1-本月，2-本年(必填)<br>")
    @PostMapping("/entryCountGroupByEntry")
    @SysLogger("entryCountGroupByEntry")
    public RespDTO<Map<String, Object>> entryCountGroupByEntry(@RequestBody FilterVO filterVO) {
        return RespDTO.onSuc(consoleFacade.entryCountGroupByEntry(filterVO));
    }

    /**
     * 各科室缺陷占比
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "各科室缺陷占比[by:zhaops]",
            notes = "type： 统计维度 1-本月，2-本年(必填)<br>")
    @PostMapping("/entryByDept")
    @SysLogger("entryByDept")
    public RespDTO<Map<String, Object>> entryByDept(@RequestBody FilterVO filterVO) {
        return RespDTO.onSuc(consoleFacade.entryByDept(filterVO));
    }
    //-----------------------单独接口结束-------------------------------
}